@@ -10,7 +10,7 @@ import mobster.utils_mobster as mut
 
 
 @config_enumerate
-def guide(data, K=1, tail=1, purity=0.96, clonal_beta_var=1., alpha_prior_concentration = 5, alpha_prior_rate = 0.1, number_of_trials_clonal_mean=100.,
+def guide(data, K=1, tail=1, purity=0.96, clonal_beta_var=1., number_of_trials_clonal_mean=100.,
           number_of_trials_clonal=900., number_of_trials_k=300., prior_lims_clonal=[1., 10000.],alpha_precision_concentration = 5, alpha_precision_rate=0.1,
           prior_lims_k=[1., 10000.]):
     """
@@ -38,7 +38,6 @@ def guide(data, K=1, tail=1, purity=0.96, clonal_beta_var=1., alpha_prior_concen
     index_1 = [i for i, j in enumerate(theoretical_num_clones) if j == 1]
 
     a_prior = pyro.param("tail_mean", torch.tensor(1.5), constraint=constraints.interval(0.5, 5))
-    sd_prior = pyro.param("tail_sd",dist.Gamma(concentration=alpha_prior_concentration, rate=alpha_prior_rate).mean, constraint=constraints.positive)
     alpha_precision_par = pyro.param("alpha_noise",
                                  dist.Gamma(concentration=alpha_precision_concentration, rate=alpha_precision_rate).mean * torch.ones([len(karyos)]),
                                  constraint=constraints.positive)
@@ -47,9 +46,6 @@ def guide(data, K=1, tail=1, purity=0.96, clonal_beta_var=1., alpha_prior_concen
 
 
 
-    alpha_prior_sd = pyro.sample('sd_tail', dist.Delta(sd_prior))
-
-
     alpha_prior = pyro.sample('u', dist.Delta(a_prior))
 
     idx1 = 0
@@ -65,12 +61,30 @@ def guide(data, K=1, tail=1, purity=0.96, clonal_beta_var=1., alpha_prior_concen
         weights_param_1 = pyro.param("param_weights_1", (1 / (K + 1)) * torch.ones([len(index_1), K + 1]),
                                      constraint=constraints.simplex)
 
+    if K != 0:
+        if 2 in counts_clones.keys():
+
+            k2 = torch.cat([theoretical_clonal_means[kr] for kr in range(len(karyos)) if theoretical_num_clones[kr] == 2]).reshape([counts_clones[2],2])
+
+
+            k_means_2 = ((torch.amin(k2, dim = 1) - 0.12) / (K +1) ).reshape([1, counts_clones[2]]) * torch.arange(1, K + 1).reshape([K,1]) + 0.12
+        else:
+            k_means_2 = 0
+
+
+        if 1 in counts_clones.keys():
+            k1 = torch.tensor([theoretical_clonal_means[kr] for kr in range(len(karyos)) if theoretical_num_clones[kr] == 1]).reshape([counts_clones[1],1])
+
+            k_means_1 = ((torch.amin(k1, dim = 1) - 0.12) / (K + 1)).reshape([1,counts_clones[1]]) * torch.arange(1, K + 1).reshape([K, 1])  + 0.12
+        else:
+            k_means_1 = 0
+    else:
+        k_means_1 = k_means_2 = 0
+
+
+
     for kr in pyro.plate("kr", len(karyos)):
 
-        alpha_precision = pyro.sample('alpha_precision_{}'.format(kr), dist.Delta(alpha_precision_par[kr]))
-
-        alpha_teo = pyro.sample('alpha_{}'.format(kr), dist.LogNormal(torch.log(alpha_prior) * mut.theo_allele_list[karyos[kr]], 1/alpha_prior_sd))
-
 
 
         if theoretical_num_clones[kr] == 2:
@@ -79,19 +93,33 @@ def guide(data, K=1, tail=1, purity=0.96, clonal_beta_var=1., alpha_prior_concen
 
             # Mean parameter when the number of clonal picks is 2
             a_2_theo = torch.cat([theoretical_clonal_means[i] for i in index_2]).reshape([2, counts_clones[2]])
-            max_lim_2 = torch.tensor([torch.min(theoretical_clonal_means[i]) for i in index_2])
-            a_2_k = dist.Uniform(0.1, max_lim_2 - (0.3 * max_lim_2)).sample([K])
 
             # Number of trials parameter when the number of clonal picks is 2
             b_2_theo = torch.ones([2, len(index_2)]) * number_of_trials_clonal_mean
-            b_2_k = torch.ones([K, len(index_2)]) * number_of_trials_k
 
-            a21 = pyro.param('a_2',
-                             torch.cat((a_2_theo, a_2_k)).reshape([2 + K, len(index_2)]),
-                             constraint=constraints.unit_interval)
-            a22 = pyro.param('b_2',
-                             torch.cat((b_2_theo, b_2_k)).reshape([2 + K, len(index_2)]),
-                             constraint=constraints.positive)
+            # If we have subclones
+            a21 = a22 = 0
+            if K != 0:
+                # mean of the subclones, located more or less between the smallest clonal cluster and the tail
+                a_2_k = dist.Uniform(k_means_2 - 0.05, k_means_2 + 0.05).sample().reshape([K, counts_clones[2]])
+                # Number of trials  for the subclones
+                b_2_k = torch.ones([K, len(index_2)]) * number_of_trials_k
+
+                a21 = pyro.param('a_2',
+                                 torch.cat((a_2_theo, a_2_k)).reshape([2 + K, len(index_2)]),
+                                 constraint=constraints.unit_interval)
+                a22 = pyro.param('b_2',
+                                 torch.cat((b_2_theo, b_2_k)).reshape([2 + K, len(index_2)]),
+                                 constraint=constraints.positive)
+            else:
+                a21 = pyro.param('a_2',
+                                 a_2_theo.reshape([2 + K, len(index_2)]),
+                                 constraint=constraints.unit_interval)
+                a22 = pyro.param('b_2',
+                                 b_2_theo.reshape([2 + K, len(index_2)]),
+                                 constraint=constraints.positive)
+
+
 
             with pyro.plate("clones_{}".format(kr), 2 + K):
                 pyro.sample('beta_clone_mean_{}'.format(kr), dist.Delta(a21[:, idx2]))
@@ -103,19 +131,30 @@ def guide(data, K=1, tail=1, purity=0.96, clonal_beta_var=1., alpha_prior_concen
 
             # Mean parameter when the number of clonal picks is 1
             a_1_theo = torch.tensor([theoretical_clonal_means[i] for i in index_1]).reshape([1, counts_clones[1]])
-            max_lim_1 = torch.tensor([np.min(theoretical_clonal_means[i]) for i in index_1])
-            a_1_k = dist.Uniform(0.1, max_lim_1 - (0.3 * max_lim_1)).sample([K])
 
             # Number of trials parameter when the number of clonal picks is 1
             b_1_theo = torch.ones([1, len(index_1)]) * number_of_trials_clonal_mean
-            b_1_k = torch.ones([K, len(index_1)]) * number_of_trials_k
 
-            a11 = pyro.param('a_1',
-                             torch.cat((a_1_theo, a_1_k)).reshape([1 + K, len(index_1)]),
-                             constraint=constraints.unit_interval)
-            a12 = pyro.param('b_1',
-                             torch.cat((b_1_theo, b_1_k)).reshape([1 + K, len(index_1)]),
-                             constraint=constraints.positive)
+            if K != 0:
+
+                a_1_k = dist.Uniform(k_means_1 - 0.05, k_means_1 + 0.05).sample().reshape([K, counts_clones[1]])
+                b_1_k = torch.ones([K, len(index_1)]) * number_of_trials_k
+
+                a11 = pyro.param('a_1',
+                                 torch.cat((a_1_theo, a_1_k)).reshape([1 + K, len(index_1)]),
+                                 constraint=constraints.unit_interval)
+                a12 = pyro.param('b_1',
+                                 torch.cat((b_1_theo, b_1_k)).reshape([1 + K, len(index_1)]),
+                                 constraint=constraints.positive)
+            else:
+
+
+                a11 = pyro.param('a_1',
+                                 a_1_theo.reshape([1 + K, len(index_1)]),
+                                 constraint=constraints.unit_interval)
+                a12 = pyro.param('b_1',
+                                 b_1_theo.reshape([1 + K, len(index_1)]),
+                                 constraint=constraints.positive)
 
             with pyro.plate("clones_{}".format(kr), 1 + K):
                 pyro.sample('beta_clone_mean_{}'.format(kr), dist.Delta(a11[:, idx1]))
@@ -125,8 +164,9 @@ def guide(data, K=1, tail=1, purity=0.96, clonal_beta_var=1., alpha_prior_concen
         if tail == 1:
             # K = K + tail
             pyro.sample('weights_tail_{}'.format(kr), dist.Delta(weights_tail[kr]).to_event(1))
+            alpha_precision = pyro.sample('alpha_precision_{}'.format(kr), dist.Delta(alpha_precision_par[kr]))
 
             with pyro.plate('data_{}'.format(kr), len(data[karyos[kr]])):
                 pyro.sample("alpha_noise_{}".format(kr),
-                            dist.LogNormal(torch.log(alpha_teo), 1 / alpha_precision))
+                            dist.LogNormal(torch.log(alpha_prior) * mut.theo_allele_list[karyos[kr]], 1 / alpha_precision))
 
